package com.iflytek.cloud;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * <h3>识别结果</h3>
 * <p>
 * 		识别结果为文本内容，格式包括自然语言文本(plain)、xml结构化文本(xml)、json结构化文本(json)3种。
 * </p>
 * <p>
 * 		当使用听写（{@link SpeechConstant#CLOUD_GRAMMAR}为null）时，结果文本格式支持：plain、json（默认）。
 * </p>
 * <p>
 * 		当使用语法和命令词识别（{@link SpeechConstant#CLOUD_GRAMMAR}为非null）时，结果文本格式支持:plain、xml、json（默认）。
 * </p>
 * <p>
 * 		当使用{@link SpeechUnderstander 语义理解}时，结果文本格式支持:json、xml。
 * (若{@link SpeechConstant#NLP_VERSION}值为1.0，则只支持xml格式)
 * </p>
 * <p>
 * 		关于json和xml格式的各关键字说明，请参考<script>gShowUserGuideFile();</script>。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechRecognizer
 * @see RecognizerListener#onResult
 * @see SpeechUnderstander
 * @see SpeechConstant#RESULT_TYPE
 * @see SpeechConstant#CLOUD_GRAMMAR
 */
public class RecognizerResult implements Parcelable {
	/**
	 * 识别、转写、语义结果，请参考文档中识别结果定义。
	 */
	private String mResult = "";

	/**
	 * <h4>构造函数</h4>
	 * <p>应用层只通过{@link RecognizerListener#onResult}得到结果，可忽略此类的构造函数。</p>
	 * @param in 文本结果
	 * @hide 
	 */
	public RecognizerResult(Parcel in) {
		mResult = in.readString();
	}

	/**
	 * <h4>构造函数</h4>
	 * <p>应用层只通过{@link RecognizerListener#onResult}得到结果，可忽略此类的构造函数。</p>
	 * @param result 文本结果
	 */
	public RecognizerResult(String result) {
		if (null != result) {
			mResult = result;
		}
	}

	/**
	 * @hide
	 */
	public int describeContents() {
		return 0;
	}

	/**
	 * <h4>获取识别结果</h4>
	 * @return String类型识别结果文本。返回的结果文本有可能为空字符串。
	 */
	public String getResultString() {
		return mResult;
	}

	/**
	 * @hide
	 */
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeString(mResult);
	}

	/**
	 * @hide
	 */
	public static final Parcelable.Creator<RecognizerResult> CREATOR = new Parcelable.Creator<RecognizerResult>() {

		@Override
		public RecognizerResult createFromParcel(Parcel in) {
			return new RecognizerResult(in);
		}

		@Override
		public RecognizerResult[] newArray(int size) {
			return new RecognizerResult[size];
		}
	};
}
