package com.iflytek.cloud;

import com.iflytek.cloud.msc.module.SpeechInterface.ENGINE_MODE;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

import android.text.TextUtils;

/**
 * <h3>版本信息类</h3>
 * 
 * @author <script>gShowAuthor();</script>
 */
public class Version {
	private static final String BASE_VER = "4";
	private static final String JAR_VER = "1135";
	private static final String MODE_VER_OFFLINE = "6";
	private static final String MODE_VER_ONLINE = "5";
	
	private static final String SEPARATOR = ".";
	private static final String INVALID_MSC_VERSION = "0";
	
	private static String sVersion = "";
	private static String sMscVersion = INVALID_MSC_VERSION;
	
	/**
	 * <h4>获取控件版本号</h4>
	 * 
	 * @return 版本号，如：4.5.1027(控件版本号字段).1001(so库版本字段) 融合之后版本号升级为4.0 第二位
	 *         5表示v5，6表示v5+ 第三位控件版本号 第四位so库版本号
	 */
	public static String getVersion() {
		if( TextUtils.isEmpty(sVersion)
				|| INVALID_MSC_VERSION.equalsIgnoreCase(sMscVersion) ){
			sVersion = BASE_VER
					+SEPARATOR+getModeVersion()
					+SEPARATOR+JAR_VER
					+SEPARATOR+getMscVersion();
		}
		return sVersion;
	}
	
	private static String getMscVersion(){
		if( INVALID_MSC_VERSION.equalsIgnoreCase(sMscVersion) ){
			try{
				if( MSC.isLoaded() ){
					MSCSessionInfo info = new MSCSessionInfo();
					final String ver_param_name = "ver_msc";
					final byte[] ret = MSC.QMSPGetVersion( ver_param_name.getBytes(DataUtil.GB2312), info );
					if( ErrorCode.SUCCESS==info.errorcode ){
						String fullMscVer = new String( ret, DataUtil.GB2312 );
						DebugLog.LogD( "get msc full version name: "+fullMscVer );
						final int idxLastSeparator = fullMscVer.lastIndexOf( SEPARATOR );
						if( 0 <= idxLastSeparator && fullMscVer.length() > idxLastSeparator+1 ){
							sMscVersion = fullMscVer.substring( idxLastSeparator+1 );
						}
					}else{
						DebugLog.LogD( "get msc version error: "+info.errorcode );
					}
				}else{
					DebugLog.LogE( "get msc version msc is not load." );
				}
			}catch( Throwable e ){
				DebugLog.LogE( "get msc version exception:" );
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of msc version is invalid
		
		return sMscVersion;
	}
	
	private static String getModeVersion(){
		return ENGINE_MODE.MSC==SpeechUtility.DEF_ENGINE_MODE ? MODE_VER_OFFLINE : MODE_VER_ONLINE;
	}
}