package com.iflytek.cloud.msc.isr;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.concurrent.ConcurrentLinkedQueue;

import android.content.Context;
import android.media.MediaRecorder;
import android.media.MediaRecorder.AudioSource;
import android.os.Bundle;
import android.os.HandlerThread;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.RecognizerListener;
import com.iflytek.cloud.RecognizerResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.cloud.record.FilePcmRecorder;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.cloud.record.PcmRecorder.PcmRecordListener;
import com.iflytek.msc.MSC;

/**
 * Recognize thread,manager record and msc interface
 * 
 * @author iflytek
 * 
 */

public class MscRecognizer extends MscHandler implements PcmRecordListener {
	protected static final int INFINITE = -1;
	
	// 外部回调接口
	protected volatile 	 RecognizerListener mListener = null;

	// 第一次获取到录音数据
	protected boolean       mFirstRecordAudio  = false;
	//首次获取到语音写入MSC时间
	protected boolean    	mFirstDataWrite  = false;
	//获取到第一个识别结果
	protected boolean    	mFirstDataArrival  = false;
	// 是否用户主动点击了停止录音
	protected boolean    	mUstopRecord = false;
	// 录音源
	protected int           mAudioSource = AudioSource.MIC;
	
	//是否是听写会话，听写和识别设置的vad超时时间不同
	protected boolean      mLongTimeInput     = true;
	
	protected IsrSession  mSession = new IsrSession();
	protected PcmRecorder mRecorder = null;
	
	// 会话产生的上下行流量
	public static int     mUpflow =  0;
	public static int     mDownflow = 0;
	
	// 命令词识别所使用语法ID
	protected String mGramId = null;
	
	// 录音缓存队列，用于录音回放和文件保存
	protected ConcurrentLinkedQueue<byte[]> mRecordBuffer = null;
	
	/*识别结果列表,保存的为原始结果*/
	protected ArrayList<String> mResults = null;
	
	// 性能结构化日志
	protected SessPerflog mPerflog = new SessPerflog();
	
	// 需要过滤掉的音频块大小
	protected int filter_audio = 0;
	
	// onEvent回调音频数据
	private boolean notify_record_data = false;
	private String mSessionID = null;
	
	private boolean mUseBluetooth = false;
	
	public MscRecognizer(Context context,HashParam param,HandlerThread mHandlerThread) {
		super( context, mHandlerThread);
		mRecordBuffer = new ConcurrentLinkedQueue<byte[]>();
		mResults = new ArrayList<String>();
		mUstopRecord = false;
		setParams(param);
	}
	
	/**
	 * 获取录音源
	 * @return 示例：-1、MIC
	 */
	public int getAudioSource()
	{
		return mAudioSource;
	}

	/**
	 * 录音缓存队列
	 * @return
	 */
	public ConcurrentLinkedQueue<byte[]> getBuffer() {
		return mRecordBuffer;
	}

	
	/**
	 * 通知音量变化
	 * @param data
	 * @param volume
	 */
	public void callbackVolume(byte[] data, int volume) {
		if (null != mListener && isRunning())
		{
			mListener.onVolumeChanged(volume,data);
			if(notify_record_data){
				// 抛出音量、音频
				Bundle bundle = new Bundle();
				bundle.putByteArray(SpeechEvent.KEY_EVENT_RECORD_DATA,data);
				mListener.onEvent(SpeechEvent.EVENT_RECORD_DATA,volume, 0,bundle);
			}
		}
	}
	

	/**
	 * 开始识别
	 * @param listener 识别监听器
	 */
	public synchronized void startListening(RecognizerListener listener) {
		// 启动会话消息
		
        mListener = listener; 
		DebugLog.LogD("[isr]startListening called");
		// 启动会话
		start();
	}

	@Override
	protected void start() {
		mPerflog.startSession(getParam());
		super.start();
	}
	
	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		super.onMsgProcess(msg);
		
		switch( msg.what )
		{
			case MSG_START:
				proc_Msg_Start();
				break;
			case MSG_SESSION_BEGIN:
				proc_Msg_Session_Begin();
				break;
			case MSG_SPEECH_TIMEOUT:
				vadEndCall();
				break;
			case MSG_RECORD_AUDIO:
				proc_Msg_Record_Data(msg);
				break;
			case MSG_RECORD_STOPD:
				proc_Msg_Record_Stoped();
				break;
			case MSG_RESULT:
				proc_Msg_Result(msg);
				break;
			case MSG_NET_PERF:
				proc_Msg_Netperf();
				break;
			default:
				break;
			}
	}

	/**
	 * 启动消息处理
	 * @throws Exception
	 */
	protected void proc_Msg_Start() throws Exception {	
		DebugLog.LogD("[isr]start connecting");
		String engine = getParam().getString(SpeechConstant.ENGINE_TYPE);
		
		//网络检查
		boolean net_check = getParam().getBoolean(SpeechConstant.NET_CHECK, true);
		if(net_check){
			if(SpeechConstant.TYPE_CLOUD.equals(engine))
			{
				NetworkUtil.checkNetwork(mContext);
			} else if(SpeechConstant.TYPE_MIX.equals(engine) || "mixed".equals(engine)){
				//通过try catch判断网络是否连接
				try {
					NetworkUtil.checkNetwork(mContext);
				} catch (Exception e) {
					getParam().putParam(SpeechConstant.ENGINE_TYPE, SpeechConstant.TYPE_LOCAL);
				}
			}
		}
				
		int read_rate = getParam().getInt(MscKeys.KEY_RECORD_READ_RATE, PcmRecorder.READ_INTERVAL40MS);

		// 判断不是外部写录音会话
		if(mAudioSource != MscKeys.VALUE_AUDIO_EXTERNAL && isRunning()){
			DebugLog.LogD( "[isr]start  record");
			
			if(mAudioSource == MscKeys.VALUE_AUDIO_EXTERNALFILE)
			{
				//外部指定音频文件
				String path_source = getParam().getString(SpeechConstant.ASR_SOURCE_PATH);
				mRecorder = new FilePcmRecorder(getSampleRate(), read_rate, mAudioSource, path_source);
			}
			else
			{
				mUseBluetooth = getParam().getBoolean( SpeechConstant.BLUETOOTH, mUseBluetooth );
				if( mUseBluetooth ){
					startBluetooth();
				}
				
				mRecorder = new PcmRecorder(getSampleRate(), read_rate,mAudioSource);
				// 内部开启录音机时防止外部提前调用stopListening
				if(hasMessages(MSG_RECORD_STOPD))
				{
					throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
				}
			}
			
			mPerflog.pushSessionInfo(SessPerflog.KEY_REC_OPEN);
			mRecorder.startRecording(this);
			
			if( INFINITE != mSpeechTimeOut ){
				// 语音输入超时消息
				sendMsg(MSG_SPEECH_TIMEOUT, Priority.normal, false,mSpeechTimeOut);
			}
		}

		if (null != mListener && mAudioSource > MscKeys.VALUE_AUDIO_EXTERNAL){
			//内部开启录音机时才需要回调
			mListener.onBeginOfSpeech();
		}
		
		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_SSB);
		
		// 插入到消息头部，防止音频被预先处理
		sendMsg(MSG_SESSION_BEGIN,Priority.max,false,0);
	}
	
	private int mSessionRetryCount = 0;
	/**
	 * MSC SessionBegin消息处理
	 * 特别注意点：缓冲音频的写入，必须保证在SessionBegin之后
	 * @throws Exception
	 */
	protected void proc_Msg_Session_Begin() throws Exception {
		PerfLogger.appendInfo(PerfLogger.SDK_SESSION_BIGNE, null);
		int code = mSession.sessionBegin(mContext,mGramId, this);
		if(code == 0 && null != mSession.mClientID)
		{	
			//注册回调接口
			if(isRunning())
			{
				MSC.QISRRegisterNotify(mSession.mClientID, "rsltCb", "stusCb", "errCb", this);
				setStatus(Status.recording);
				if(getParam().getBoolean(SpeechConstant.ASR_NET_PERF,false))
					sendMsg(MSG_NET_PERF,Priority.max,false,0);
			}
		}else
		{
			// 延迟消息无法发送,必须通过sleep方式，否则会造成音频丢失或处理错误
			// 需要优先级放到最高，否则会造成录音消息被预先处理，造成报错
			mSessionRetryCount++;
			if(mSessionRetryCount > 40)
				throw new SpeechError(code);
			else if(isRunning())	
			{
				Thread.sleep(15);
				sendMsg(MSG_SESSION_BEGIN, Priority.max,false,0);
			}
		}
	}
	
	/**
	 * 录音数据通知消息
	 * @param msg
	 * @throws Exception
	 */
	protected void proc_Msg_Record_Data(Message msg) throws Exception{

		byte[] data = (byte[])msg.obj;
		if (data == null || data.length == 0)
			return;
		
		// 保存录音数据，用以再次识别和保存
		mRecordBuffer.add(data);
		
		uploadData(data, true);
	}
	/**
	 * 停止录音
	 * @return
	 */
	public synchronized boolean stopListening(boolean userStop) {
		DebugLog.LogD("stopRecognize, current status is :" + getStatus()+" usercancel : " +userStop);
		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_STOP);
		releaseRecord();
		// 用户主动点击录音停止
		mUstopRecord = userStop;
		sendMsg(MSG_RECORD_STOPD);
		return true;
	}
	
	/**
	 * 录音停止的通知消息
	 * @throws SpeechError
	 * @throws IOException
	 * @throws InterruptedException
	 */
	protected void proc_Msg_Record_Stoped() throws SpeechError, IOException, InterruptedException{	
	
		DebugLog.LogD( "recording stop" );
		releaseRecord();

		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_LAU);
		// upload end flag
		mSession.pushEndFlag();
		// 最低优先级消息，会被自动清除
		updateTimeoutMsg();
	}
	/**
	 * 取消识别
	 */
	@Override
	public void cancel(boolean notifyError) {
		if(notifyError && isRunning() && mListener != null){
			mListener.onError(new SpeechError(ErrorCode.ERROR_INTERRUPT));
		}
		releaseRecord();
		// 用户主动点击取消
		if(getStatus() == Status.recording){
			mUstopRecord = true;
		}
		super.cancel(notifyError);
	}
	
	/**
	 * 返回外部当前msc的网络连接状态值，lua版本中可见
	 * 100ms返回一次数据
	 */
	public void proc_Msg_Netperf()
	{
		// 100ms返回一次
		if(isRunning())
		{
			int netpref = mSession.getIntValue(TAG_NETPERF);
			if(mListener != null)
				mListener.onEvent(SpeechEvent.EVENT_NETPREF, netpref, 0,null);
			// 继续开启下一次通知
			sendMsg(MSG_NET_PERF, Priority.normal,false,100);
		}
	}
	
	@Override
	protected void onEnd(SpeechError error) {
		DebugLog.LogD("onSessionEnd");
		releaseRecord();
		mUpflow = mSession.getIntValue(TAG_UPFLOW);
		mDownflow = mSession.getIntValue(TAG_DOWNFLOW);
		getSessionID();

		// 如果无结果，判断是否需要通知10118错误,默认不返回
		if(mResults.size() <= 0 && error == null 
			&& getParam().getBoolean(SpeechConstant.ASR_NOMATCH_ERROR, true))
		{
			error = new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
		}
		//结构化日志增加应用层错误码，成功默认返回0
		if(null != error)
			mPerflog.pushSessionInfo( SessPerflog.KEY_APP_RET, error.getErrorCode(), false );
		else
			mPerflog.pushSessionInfo( SessPerflog.KEY_APP_RET, 0, false );
		
//		mLastError = new SpeechError(SpeechError.UNKNOWN,11200);
		
		// 写入录音启动时间、用户是否主动停止等消息
		mPerflog.pushSessionInfo(SessPerflog.KEY_REC_USTOP, mUstopRecord ? "1" : "0",false);
		mSession.setParam(SessPerflog.KEY_SESSIONINFO, mPerflog.getSessionInfo());

		PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);
		if(mUserCancel)
			mSession.sessionEnd("user abort");
		else if(error != null)
			mSession.sessionEnd("error" + error.getErrorCode());
		else
			mSession.sessionEnd("success");
		PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);
		
		super.onEnd(error);
		
		if(mListener != null)
		{
			if(mUserCancel)	// cancel by user
			{
				DebugLog.LogD("RecognizerListener#onCancel");
				// mListener.onCancel();
			}else	// auto exit
			{
				DebugLog.LogD("RecognizerListener#onEnd");
				
				if(error != null) {
					// 通过onEvent抛出服务端会话id
					Bundle bundle = new Bundle();
					bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
					mListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, bundle);
					
					mListener.onError(error);
				}
			}
		}
		mListener = null;
	}

	/**
	 * called from VAD check
	 */
	public void vadEndCall() {
		if (Status.recording == getStatus()) 
		{
			DebugLog.LogD( "isr recognize vadEndCall" );
			//确保onEndOfSpeech在EVENT_RECORD_STOP事件之前被调用，从而onSpeechEnd在onRecordEnd之前被调用
			if (mListener != null)
				mListener.onEndOfSpeech();
			stopListening(false);
		}
	}

	/**
	 * 通知外部识别结果
	 * @param isLast
	 * @throws SpeechError
	 * @throws UnsupportedEncodingException
	 */
	private void notifyResult(boolean isLast, byte[] resultByte) throws SpeechError, UnsupportedEncodingException{
		String result = null;
		mStatusBegin = SystemClock.elapsedRealtime();
		if(resultByte != null && resultByte.length > 0)
		{
			result = new String(resultByte, DataUtil.UTF8);
		}else if(mResults.size() <= 0)
		{
			// 本地识别无匹配结果时v5+给出空结果,返回 ERROR_NO_MATCH
			String localGrammar  = getParam().getString(SpeechConstant.LOCAL_GRAMMAR);
			if (!TextUtils.isEmpty(localGrammar) && !"sms.irf".equals(localGrammar)) {
				throw new SpeechError(ErrorCode.ERROR_NO_MATCH);
			}else {
				if( getParam().getBoolean(SpeechConstant.ASR_NOMATCH_ERROR, true))
					throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
				else {
					result = "";
				}
			}
		}else 
		{
			result = "";
		}

		
		// 保存识别结果,用以判断是否返回10118错误
		mResults.add(result);
		if(null != mListener && isRunning())
		{
			// 需要放在结果回调前面，确保回调最后结果时已经拿到sessionID
			// 服务端会话id
			Bundle bundle = new Bundle();
			bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
			mListener.onEvent(SpeechEvent.EVENT_SESSION_ID,0, 0,bundle);
		
			if (isLast && getParam().getBoolean(MscKeys.REQUEST_AUDIO_URL,false)) {
				// 返回服务端音频url
				Bundle urlBundle = new Bundle();
				urlBundle.putString(SpeechEvent.KEY_EVENT_AUDIO_URL, mSession.getAudioUrl());
				mListener.onEvent(SpeechEvent.EVENT_AUDIO_URL,0, 0,urlBundle);
			}
			
			// 原始结果
			RecognizerResult reresult = new RecognizerResult(result);
			PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
			mListener.onResult(reresult,isLast);
		}
		
		DebugLog.LogD("msc result time:" + System.currentTimeMillis());
		
		if(isLast)
		{
			exit(null);
		}
	}
	
	/**
	 * 得到识别结果，不进行解析
	 * @return
	 */
	public RecognizerResult getRecognizeResult() throws SpeechError
	{
		RecognizerResult result = null;
		try {
			if(mSession.getResultData() != null)
			{
				result = new RecognizerResult(new String(mSession.getResultData(),DataUtil.UTF8));
			}
		} catch (UnsupportedEncodingException e) {
			DebugLog.LogE(e);
			throw new SpeechError(ErrorCode.ERROR_INVALID_RESULT);
		}
		return result;
	}
	
	
	@Override
	protected void onParseParam() {
		mGramId = getParam().getString(SpeechConstant.CLOUD_GRAMMAR);

		mAudioSource =  getParam().getInt(SpeechConstant.AUDIO_SOURCE, MediaRecorder.AudioSource.MIC);

		mLongTimeInput = ParamBuilder.isLongInput(getParam().getString(SpeechConstant.DOMAIN));

		int filterTime = getParam().getInt(SpeechConstant.FILTER_AUDIO_TIME, 0);
		filter_audio = getParam().getInt(SpeechConstant.SAMPLE_RATE, mSampleRate)/1000*16/8*filterTime;
		
		// 语音最长输入时间
		mSpeechTimeOut = getParam().getInt(SpeechConstant.KEY_SPEECH_TIMEOUT, mSpeechTimeOut);
		
		// 通过onEvent 抛出音频数据
		notify_record_data =  getParam().getBoolean(SpeechConstant.NOTIFY_RECORD_DATA, false);
				
		DebugLog.LogD("mSpeechTimeOut=" + mSpeechTimeOut);
		super.onParseParam();
	}
	
	@Override
	public boolean isLongInput() {
		return mLongTimeInput;
	}
	
	/**
	 * 状态消息通知
	 * @param clientSessionID
	 * @param type
	 * @param status
	 * @param param
	 * @param param2
	 */
	void stusCb(char[] clientSessionID, int type, int status, int param, byte[] param2)
	{
//		DebugLog.LogE("MscRecognizer", "stusCb:"+status+",type:"+type);
		if (type == 0 && status == 3) {
			// VAD后端点消息
			DebugLog.LogD("MscRecognizer", "stusCb:"+status+",type:"+type);
			vadEndCall();
		}
	}
	/**
	 * msc识别结果回调接口
	 * 
	 * @param clientSessionID
	 * @param result
	 * @param len
	 * @param resultstatus
	 */
	void rsltCb(char[] clientSessionID, byte[] result, int len, int resultstatus)
	{
		// 识别结果回调消息，优先级最高
		if(result != null)
			DebugLog.LogD("MscRecognizer", "rsltCb:"+resultstatus + "result:" + new String( result ));
		else
			DebugLog.LogI("MscRecognizer", "rsltCb:"+resultstatus + "result:null");
		Message msg = obtainMessage(MSG_RESULT, resultstatus, 0, result);
		//消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
		//ps:概率很小
		if(hasMessages( MSG_RESULT ))
			sendMsg(msg, Priority.normal, false, 0);
		else
			sendMsg(msg, Priority.max, false, 0);
	}
	
	/**
	 * 错误回调消息
	 * @param clientSessionID
	 * @param errorcode
	 * @param detailDesc
	 */
	void errCb(char[] clientSessionID, int errorcode, byte[] detailDesc)
	{
		onError(new SpeechError(errorcode));
	}

	/**
	 * stop listen message
	 * @throws SpeechError 
	 * @throws IOException 
	 * @throws InterruptedException 
	 */

	protected void releaseRecord()
	{
		// stop audio record
		if (mRecorder != null){
			mRecorder.stopRecord(getParam().getBoolean(MscKeys.RECORD_FORCE_STOP, false));
			mRecorder = null;
			mPerflog.pushSessionInfo(SessPerflog.KEY_REC_CLOSE);
			//notify record will stop 
			if(null != mListener)
			{
				mListener.onEvent(SpeechEvent.EVENT_RECORD_STOP, 0, 0, null);
			}
			
			if( mUseBluetooth ){
				stopBluetooth();
			}
		}
	}
	/**
	 * 结果通知消息
	 * 	status:   2----unknown 0----has result 5----is last result
	 * @throws SpeechError
	 * @throws InterruptedException
	 * @throws UnsupportedEncodingException
	 */
	void proc_Msg_Result(Message msg) throws SpeechError, InterruptedException, UnsupportedEncodingException{
		int status = msg.arg1;
		byte[] data = (byte[])msg.obj;
		//long start = SystemClock.elapsedRealtime();
//		long t=SystemClock.elapsedRealtime();
		switch (status) {
			case 2://回调版本msc不返回2
				break;
			case 0:
				if (!mFirstDataArrival) {
					mFirstDataArrival = true;
					mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FRS);
				}
				notifyResult(false, data);
				break;
			case 5:
				if (!mFirstDataArrival) {
					mFirstDataArrival = true;
					mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FRS);
				}
				mPerflog.pushSessionInfo(SessPerflog.KEY_APP_LRS);
				notifyResult(true, data);
				break;
			default:
				break;
		}
		//DebugLog.LogD( "get result time is"+(SystemClock.elapsedRealtime()-start) );
	}
	
	/**
	 * 传入音频
	 * @param data
	 * @param needvadNotify
	 * @throws SpeechError
	 */
	protected void uploadData(byte[] data,boolean needvadNotify) 
		throws SpeechError{	

		// 第一次向msc写入音频时间
		if (false == mFirstDataWrite) {
			mFirstDataWrite = true;
			mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FAU);
			//通知外部开始有音频数据
			if(null != mListener)
			{
				mListener.onEvent(SpeechEvent.EVENT_SPEECH_START, 0, 0, null);
			}
		}

		mSession.pushAudioData(data, data.length);

		if (needvadNotify) {
			int volume = mSession.getAudioVolume();
			DebugLog.LogI("QISRAudioWrite volume:" + volume);
			callbackVolume(data, volume);
		}
	}
	
	public int writeAudio( byte[] data,int offset,int length ){
		onRecordBuffer( data, offset, length );
		return ErrorCode.SUCCESS;
	}

	/**
	 * record callback from PcmRecorder
	 * 
	 * @throws IOException
	 */
	@Override
	public void onRecordBuffer(byte[] data,int offset,int length) {
		// record data notify
		if(data == null || length <= 0 || data.length < length)
			return;
		if (length > 0 && isRunning()) {
			
			if(false == mFirstRecordAudio){
				mFirstRecordAudio = true;
				mPerflog.pushSessionInfo(SessPerflog.KEY_REC_START);
			}

			if (filter_audio > 0) { //需要过滤掉的音频块大小
				if (filter_audio >= length) {
					filter_audio -= length;
				}else {
					// 拷贝内存，发送录音消息
					byte[] temp = new byte[length-filter_audio];
					System.arraycopy(data, offset+filter_audio, temp, 0, length-filter_audio);
					sendMsg(obtainMessage(MSG_RECORD_AUDIO,temp));
					filter_audio = 0;
				}
			}else {
				// 拷贝内存，发送录音消息
				byte[] temp = new byte[length];
				System.arraycopy(data, offset, temp, 0, length);
				sendMsg(obtainMessage(MSG_RECORD_AUDIO,temp));
			}
		}
	}

	/**
	 * 录音错误
	 */
	@Override
	public void onError(SpeechError error) {
		//记录应用层错误码
		exit(error);
	}

	/**
	 * 录音启动
	 */
	@Override
	public void onRecordStarted(boolean success) {
		mPerflog.pushSessionInfo(SessPerflog.KEY_REC_READY);
	}
	
	/**
	 * 返回结构化日志对象，便于外部写入
	 * @return
	 */
	public SessPerflog getPerfLog()
	{
		return mPerflog;
	}

	/**
	 * 录音机释放通知
	 */
	@Override
	public void onRecordReleased() {
		if(mRecorder != null && mRecorder instanceof FilePcmRecorder)
		{
			//文件录音释放录音机则有可能是文件读完，模拟手动停止录音
			stopListening(true);
		}
	}

	/**
	 * 返回服务端ID
	 */
	@Override
	public String getSessionID()
	{
		if( TextUtils.isEmpty(mSessionID) ){
			mSessionID = mSession.getSessionID();
		}
		return mSessionID;
	}
	/**
	 * 返回客户端ID
	 */
	@Override
	public String getClientID()
	{
		return mSession.getClientID();
	}

}