package com.iflytek.cloud.msc.isv;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;

import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechListener;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;

/**
 * 声纹模型管理类，负责模型的异步查询和删除，使用JSON返回操作结果给listener。
 * @author jianhuang3</a>
 *
 */
public class ModelManager extends MscHandler{
	private VerifySession mSession = new VerifySession();
	private SpeechListener mInnerListener;

	public ModelManager(Context context, HandlerThread handlethread) {
		super(context, handlethread);
	}
	
	public void sendRequest(HashParam param, InnerSpeechListener listener) {
		mInnerListener = listener;
		setParams(param);
		Message msg = Message.obtain();
		msg.what = MSG_ISV_REQUEST;
		
		sendMsg(msg);
	}
	
	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		switch (msg.what) {
		case MSG_ISV_REQUEST:
			synchronized (ModelManager.class) {
				int ret = mSession.sendRequest(mContext, ModelManager.this);
				String result = String.format("{'ret':%d,'cmd':%s}", 
						ret, getParam().getString(SpeechConstant.ISV_CMD));
				
				if (null != mInnerListener) {
					mInnerListener.onBufferReceived(result.getBytes(getParamEncoding()));
				}
				
				// 结束HandlerThread
				sendMsg(MSG_END);
			}
			break;

		default:
			break;
		}
	}
	
	@Override
	protected void onEnd(SpeechError error) {
		super.onEnd(error);
		if(mInnerListener != null) {
			mInnerListener.onCompleted(error);
		} 
	}

	@Override
	public String getSessionID() {
		return mSession.getSessionID();
	}

	@Override
	public String getClientID() {
		return mSession.getClientID();
	}
	
	private static final int BASE_MSG_EVENT = 0;
	private static final int BASE_MSG_DATA = 1;
	private static final int BASE_MSG_END = 2;

	public final static class InnerSpeechListener implements SpeechListener {
		private SpeechListener mOutListener = null;
		
		public InnerSpeechListener(SpeechListener listener) {
			mOutListener = listener;
		}
		
		private Handler mUiHandler = new Handler(Looper.getMainLooper()){
			public void handleMessage(Message msg) {
				if (null == mOutListener) {
					return;
				}
				switch (msg.what) {
				case BASE_MSG_EVENT:
					mOutListener.onEvent(msg.arg1, (Bundle)msg.obj);
					break;
				case BASE_MSG_DATA:
					mOutListener.onBufferReceived((byte[])msg.obj);
					break;
				case BASE_MSG_END:
					mOutListener.onCompleted((SpeechError) msg.obj);
					break;

				default:
					break;
				}
			}
		};

		@Override
		public void onEvent(int eventType, Bundle params) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_EVENT, eventType,
					0, params);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBufferReceived(byte[] buffer) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_DATA, buffer);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onCompleted(SpeechError error) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_END, error);
			mUiHandler.sendMessage(msg);
		}
		
	}

}
