package com.iflytek.cloud.msc.tts;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.os.Build;
import android.os.HandlerThread;
import android.os.Message;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.record.PcmBuffer;

public class MscSynthesizer extends MscHandler{
	
	/** member properties */
	private String mText = "";
	private TtsSession mSession = null;
	private MscSynthesizerListener mListener = null;
	private ArrayList<byte[]> mDatas = null;
	private int mPrePos = 0;
	private int mPrePrePos = 0;

	//whether return spell information
	private boolean isReturnSpell = false;
	
	private boolean mAudioInfoEnable = false;
	private int mCurAudioLen = 0;
	private final JSONObject mJsonInfo = new JSONObject();
	private JSONArray mJsonSpellInfo = null;

	/* 
	 * TTS合成多长时间音频后开始播放，默认-1时为第一句，如果设置为大于等于0，
	 * 表示缓冲即开始播放，但是进度不准确
	 */
	private int mBufferTime = -1;
	private int mProcScale = PcmBuffer.DEF_PROC_SCALE;
	
	public static int    mUpflow = 0;
	public static int    mDownflow = 0;	
	
	private String mSessionID = null;
	
	public MscSynthesizer(Context context,HashParam param,HandlerThread handlerthread) {
		super(context,handlerthread);
		mSession = new TtsSession();
		mDatas = new ArrayList<byte[]>();
		
		setParams(param);
	}
	
	@Override
	protected void onParseParam() {
		//音频缓冲时间
		mBufferTime = getParam().getInt(SpeechConstant.TTS_BUFFER_TIME, mBufferTime);
		mProcScale = getParam().getInt( MscKeys.KEY_TTS_PROC_SCALE, mProcScale );
//		if(mBufferTime >= 0)
//			PerfLogger.appendInfo(PerfLogger.TTS_REALTIME, null);
		super.onParseParam();
	}

	/**
	 * 开始合成会话
	 * @param text
	 * @param listener
	 */
	public void startSyn(String text, MscSynthesizerListener listener) {
		mText = text;
		mListener = listener;
		if (text == null || TextUtils.isEmpty(text)) {
			/** invoke call back to indicate that text is empty*/
			SpeechError mLastError = new SpeechError(ErrorCode.ERROR_EMPTY_UTTERANCE);
			exit( mLastError );
			return;
		}
		//设置是否获取拼音标注信息.
		isReturnSpell = getParam().getBoolean(SpeechConstant.TTS_SPELL_INFO, false);
		mAudioInfoEnable = getParam().getBoolean( MscKeys.KEY_AUDIO_INFO, mAudioInfoEnable );
		start();
	}
	
	@Override
	public String getTextEncoding() {
		return getParam().getString(SpeechConstant.TEXT_ENCODING, DataUtil.UNICODE);
	}
	
	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		switch (msg.what) {
		case MSG_START:
			proc_Msg_Start();
			break;
		case MSG_SESSION_BEGIN:
			proc_Msg_SessionBegin();
			break;
		case MSG_TTS_AUDIO:
			proc_Msg_Tts_Audio();
			break;
		default:
			break;
		}
	}
	
	/**
	 * 处理会话启动消息
	 */
	protected void proc_Msg_Start()throws Exception {
		// check net work
		DebugLog.LogD("tts msg start:" + System.currentTimeMillis());
		String engine = getParam().getString(SpeechConstant.ENGINE_TYPE);
		boolean net_check = getParam().getBoolean(SpeechConstant.NET_CHECK, true);
		if( (SpeechConstant.TYPE_CLOUD.equals(engine) || SpeechConstant.TYPE_DISTRIBUTED.equals(engine)) 
				&& net_check ){
			NetworkUtil.checkNetwork(mContext);
		}
		
		sendMsg(MSG_SESSION_BEGIN);
	}
	
	// SessionBegin重试次数
	private int mSessionTryCount = 0;
	
	/**
	 * 处理会话启动消息，由于会产生失败，错误情况下需要多次调用，但必须有超时处理
	 */
	protected void proc_Msg_SessionBegin()throws Exception 
	{
		//DebugLog.LogD("proc_Msg_SessionBegin:"+this.getLooper().getThread().getName());
		// avoid 10129 10132 error 
		PerfLogger.appendInfo(PerfLogger.SDK_SESSION_BIGNE, null);
		int code = mSession.sessionBegin(mContext,null,this);
		if( code == 0)
		{
			byte[] srcData = mText.getBytes(getTextEncoding());
			if(DataUtil.UNICODE.equals(getTextEncoding()) ){
				//android8.1 unicode编码大小端有所变化
				if(Build.VERSION.SDK_INT >= 27) {
					byte[] tempDate = new byte[srcData.length ];
					System.arraycopy(srcData, 0, tempDate, 0, srcData.length );
					for(int i =0;i<(srcData.length)/2;i++) {
						tempDate[2*i+1] = srcData[i*2];
						tempDate[2*i] = srcData[2*i+1];
					}
					mSession.putText(tempDate);
				}else {
					// UNICODE去除两个字节的头部
					byte[] tempDate = new byte[srcData.length -2];
					System.arraycopy(srcData, 2, tempDate, 0, srcData.length -2);
					mSession.putText(tempDate);
				}
			}else{
				mSession.putText(srcData);
			}
			setStatus(Status.waitresult);
			sendMsg(MSG_TTS_AUDIO);
			// 检测网络超时
			updateTimeoutMsg();
		}else
		{
			mSessionTryCount++;
			if(mSessionTryCount > 40) 
				throw new SpeechError(code);
			else if(isRunning()){
				// 延迟发送下次会话请求与mscrecognize保持一致
				sendMsg(MSG_SESSION_BEGIN, Priority.normal,false,15);
			}
		}
	}

	/**
	 * 获取TTS音频消息，需要不断主动轮询去获取
	 * @throws Exception
	 */
	protected void proc_Msg_Tts_Audio()throws Exception 
	{
		PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
		if(!mSession.isLastResult())
		{
			byte[] buffer = mSession.getAudio();
			getSessionID();
			
			if (buffer != null && mListener != null) 
			{
				mCurAudioLen += buffer.length;
				int index = 0;
				if(Build.VERSION.SDK_INT >= 27) {
					index = mSession.getAudioIndex()/2-2;
				}else {
					index = mSession.getAudioIndex()/2-1;
				}

				if( 0 > index ){
					DebugLog.LogD( "get audio index value error: "+index );
					index = 0;
				}
				if(isReturnSpell)
				{
					String spell = mSession.getAudioInfo();
					if(!TextUtils.isEmpty(spell))
					{
						if( null == mJsonSpellInfo ){
							mJsonSpellInfo = new JSONArray();
						}
						
						mJsonSpellInfo.put( spell );
					}
				}

				//当音频缓存时间小于0时，应用层没有设置马上播放，等小句结束时再送出音频
				if ( 0>mBufferTime 
						&& mPrePos != 0 
						&& index != mPrePos 
						&& mDatas.size() > 0 )
				{
					DebugLog.LogI("tts msc get audio beg=" + mPrePrePos + ", end=" + mPrePos);
					notifyBuffer();
				}
				// 检测下一次超时
				updateTimeoutMsg();
				mPrePos = index;
				mDatas.add(buffer);
				
				//当音频缓存时间大于等于0时，马上送出音频
				if ( 0 <= mBufferTime ){
					notifyBuffer();
				}
				
				sendMsg(MSG_TTS_AUDIO, Priority.normal, false, 0);
			}else{
				sendMsg(MSG_TTS_AUDIO, Priority.normal, false,10);
			}
		}else  {
			DebugLog.LogD( "tts msc get last audio" );
			if (null != mListener)
			{
				if( mAudioInfoEnable ){
					this.mJsonInfo.put( MscKeys.JSON_KEY_AUDIOLEN, this.mCurAudioLen );
				}
				
				if( null != mJsonSpellInfo ){
					mJsonInfo.put( MscKeys.JSON_KEY_SPELLINFO, mJsonSpellInfo );
					mJsonSpellInfo = null;
				}
				
				mListener.onBufferProgress( mDatas, mProcScale, mPrePrePos
						, mText.length()-1
						, mJsonInfo.length()>0 ? mJsonInfo.toString() : null );
			}
			exit(null);
		}
	}
	@Override
	public void cancel(boolean notifyError) {
		if(notifyError && isRunning() && mListener != null){
			mListener.onCompleted(new SpeechError(ErrorCode.ERROR_INTERRUPT));
		}
		
		super.cancel(notifyError);
	}
	/**
	 * 结束消息
	 */
	@Override
	protected void onEnd(SpeechError error) {
		//DebugLog.LogD("onEnd:"+this.getLooper().getThread().getName());
		mUpflow = mSession.getIntValue(TAG_UPFLOW);
		mDownflow = mSession.getIntValue(TAG_DOWNFLOW);
		
		// 增加会话日志的sessionID
		getSessionID();
		
		PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);
		if(mListener == null)
			mSession.sessionEnd("user abort");
		else if(error != null)
		{
			mSession.sessionEnd("error" + error.getErrorCode());
			DebugLog.LogD("QTts Error Code = " + error.getErrorCode());
		}
		else
			mSession.sessionEnd("success");
		PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);
		
		super.onEnd(error);
		
		if(mListener != null)
		{
			if(mUserCancel)	// cancel by user
			{
				DebugLog.LogD("MscSynthesizer#onCancel");
//				mListener.onCancel();
			}else	// auto exit
			{
				DebugLog.LogD("MscSynthesizer#onEnd");
				mListener.onCompleted(error);
			}
		}
	}

	/**
	 * 返回客户端ID
	 */
	public String getClientID() {
		return mSession.getClientID();
	}
	/**
	 * 返回服务端ID
	 */
	public String getSessionID(){
		if( TextUtils.isEmpty(mSessionID) ){
			mSessionID = mSession.getSessionID();
		}
		return mSessionID;
	}

	private void notifyBuffer() throws SpeechError, JSONException{
		int percent = Math.min( mProcScale-1, mPrePos*mProcScale/mText.length() );
		if( mAudioInfoEnable ){
			this.mJsonInfo.put( MscKeys.JSON_KEY_AUDIOLEN, this.mCurAudioLen );
		}
		
		if( null != mJsonSpellInfo ){
			mJsonInfo.put( MscKeys.JSON_KEY_SPELLINFO, mJsonSpellInfo );
			mJsonSpellInfo = null;
		}
		
		mListener.onBufferProgress( mDatas, percent, mPrePrePos
				, mPrePos
				, mJsonInfo.length()>0 ? mJsonInfo.toString() : null );
		
		mDatas = new ArrayList<byte[]>();
		mPrePrePos = Math.min( mPrePos+1, mText.length()-1 );
	}

}
