package com.iflytek.cloud.msc.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.interfaces.RSAPublicKey;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import com.iflytek.cloud.msc.util.log.DebugLog;

public class Encrypter {
	private static final int SDK_VERSION_KITKAT = 19;
	private static final String RSA_PUBLIC_KEY = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCt8H0BF3SquJmk6xIo2bTldgvtazLIeSbR4cle\n" +
			"p7zeUAtI/mC7UgFl8xXFCTemVambyQFnM5GsZOI1BpAMJO7N/YHRX7hvCZG6D0rEXQEdKXhKFIBQ\n" +
			"mOYRYZP042vWRcKZ6iQLdLYmyg6tIzjYVfH0f6YX8OLIU7fy0TA/c88rzwIDAQAB";
	/**
	 * 采用简单异或方式
	 * @param data
	 * @return
	 */
	public static byte[] lightcode(byte[] data)
	{
		if(data == null)
			return null;
		for(int i = 0; i < data.length; i++)
		{
			data[i] = (byte) (data[i]^GZIP_KEY);
		}
		return data;
	}
	

	/**
	 * 对字节数组data进行GZIP压缩并进行加密
	 * 顺序：先压缩后加密
	 * 加密规则：数据按字节异或key
	 * @param data
	 * @return
	 */
	public static byte[] zip5xEncode(byte[] data)
	{
		if(data == null)
			return null;
		byte[] encodeBytes = null;
		ByteArrayOutputStream byteOut= new ByteArrayOutputStream();
		GZIPOutputStream  gzipOut	 = null;
		try 
		{
			gzipOut = new GZIPOutputStream(byteOut);
			gzipOut.write(data);
			gzipOut.finish();
			if(android.os.Build.VERSION.SDK_INT<SDK_VERSION_KITKAT){
				gzipOut.flush();
			}

//			gzipOut.flush();
			encodeBytes = byteOut.toByteArray();
			
			// 异或
			for(int i = 0; i < encodeBytes.length; i++)
			{
				encodeBytes[i] = (byte) (encodeBytes[i]^GZIP_KEY);
			}
		}
		catch (IOException e) 
		{
			DebugLog.LogE(e);
		}
		finally
		{
			try 
			{
				if(gzipOut != null)
				{		
					gzipOut.close();
				}
				if(byteOut != null)
				{
					byteOut.close();
				}
			}
			catch (IOException e) 
			{
				DebugLog.LogE(e);
			}
		}
		return encodeBytes;
	}
	
	public final static int 	BUFFER_SIZE = 1024;
	public static final byte    GZIP_KEY = 0x05;
	/**
	 * 对字节数组data进行解密并进行GZIP解压
	 * 顺序：先解密再解压
	 * 解密规则：数据按字节异或key
	 * @param data
	 * @return
	 */
	public static byte[] zip5xDecode(byte[] data)
	{
		if(data == null)
			return null;
		
		byte[] decodeBytes = null;
		
		// 异或
		for(int i = 0; i < data.length; i++)
		{
			data[i] = (byte) (data[i]^GZIP_KEY);
		}

		ByteArrayInputStream byteIn = new ByteArrayInputStream(data);
		GZIPInputStream gzipIn = null;
		ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
		try 
		{  
			gzipIn = new GZIPInputStream(byteIn);
        	byte[] buffer = new byte[BUFFER_SIZE];
        	int count = -1;
            while ((count = gzipIn.read(buffer,0,buffer.length)) != -1) 
            {  
            	byteOut.write(buffer, 0, count);
            }  
            decodeBytes = byteOut.toByteArray();    
            byteOut.flush();
		}
		catch (IOException e) 
		{
			DebugLog.LogE(e);
		}
		finally
		{
			try 
			{
				if(gzipIn != null)
				{		
					gzipIn.close();
				}
				if(byteIn != null)
				{
					byteIn.close();
				}
				if(byteOut != null)
				{
					byteOut.close();
				}
			}
			catch (IOException e) 
			{
				DebugLog.LogE(e);
			}
		}
		return decodeBytes;
	}
	
	/*
	 * MD5加密，生成32位字符串，线程同步.
	 * @param str 原始字符串
	 */
    public synchronized static String MD5(String str)  
    {  
        MessageDigest md5 = null;  
        try  
        {  
            md5 = MessageDigest.getInstance("MD5"); 
            
	        char[] charArray = str.toCharArray();  
	        byte[] byteArray = new byte[charArray.length];  
	          
	        for(int i = 0; i < charArray.length; i++)  
	        {  
	            byteArray[i] = (byte)charArray[i];  
	        }  
	        byte[] md5Bytes = md5.digest(byteArray);  
	          
	        StringBuffer hexValue = new StringBuffer();  
	        for( int i = 0; i < md5Bytes.length; i++)  
	        {  
	            int val = ((int)md5Bytes[i])&0xff;  
	            if(val < 16)  
	            {  
	                hexValue.append("0");  
	            }  
	            hexValue.append(Integer.toHexString(val));  
	        }  
	        return hexValue.toString();  
        }catch(Exception e)  
        {  
            DebugLog.LogE(e);  
            return "";  
        } 
    }

    //取偶数位的MD5值
    public synchronized static String cut16MD5(String str) {
		DebugLog.LogD("cut16MD5 start:"+str);
		String md5_str = MD5(str);
		DebugLog.LogD("cut16MD5 start:"+md5_str);
		StringBuffer result = new StringBuffer();
		if (md5_str == null || md5_str.length() == 0){
			return null;
		}
		DebugLog.LogD("cut16MD5 md5 size is:"+md5_str.length());
		for( int i = 0;i<md5_str.length();i++) {
			if (i%2 == 0){
				DebugLog.LogD("cut16MD5 result i:"+md5_str.charAt(i));
				result.append(md5_str.charAt(i));

			}
			DebugLog.LogD("cut16MD5 result i = :"+i);
		}
		DebugLog.LogD("cut16MD5 result:"+result);
		return result != null?result.toString():null;
	}
    
    /**
	 * 获取文件的MD5
	 * @param file
	 * @return
	 */
	public static String getFileMd5(File file) {
		String value = null;
		if (!file.exists()) {
			return null;
		}
		
		FileInputStream in = null;
		try {
			in = new FileInputStream(file);
			MappedByteBuffer byteBuffer = in.getChannel().map(
					FileChannel.MapMode.READ_ONLY, 0, file.length());
			MessageDigest md5 = MessageDigest.getInstance("MD5");
			md5.update(byteBuffer);
			BigInteger bi = new BigInteger(1, md5.digest());
			value = bi.toString(16);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (null != in) {
				try {
					in.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return value;
	}

	private static byte[] zip(byte[] data) {
		if (data == null) {
			return null;
		}
		byte[] encodeBytes = null;
		ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
		GZIPOutputStream gzipOut = null;
		try {
			gzipOut = new GZIPOutputStream(byteOut);
			gzipOut.write(data);
			gzipOut.finish();
			if (android.os.Build.VERSION.SDK_INT < SDK_VERSION_KITKAT) {
				gzipOut.flush();
			}
			encodeBytes = byteOut.toByteArray();

		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (gzipOut != null) {
					gzipOut.close();
				}
				byteOut.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return encodeBytes;
	}


	public static byte[] encrypt(byte[] heart) {
		try {
			byte[] key = AESUtil.generateKey().getEncoded();

			byte[] encryptHeart = null;
			if (heart != null) {
				encryptHeart = AESUtil.encrypt(zip(heart), key);
			}

			RSAPublicKey rsaKey = RSAUtil.loadPublicKey(RSA_PUBLIC_KEY);
			byte[] encryptKey = RSAUtil.encryptByPublicKey(key, rsaKey);

			//秘钥为空，返回
			if (encryptKey == null || encryptHeart == null) {
				return null;
			}

			int keyLen = encryptKey.length;
			int heartLen = encryptHeart.length;

			//TLV格式，type 1字节, len 4字节，需要额外增加2*(4+1)字节
			byte[] encode = new byte[keyLen + heartLen + 10];

			//type key，1字节， 0x01
			encode[0] = 0x01;
			//偏移量
			int offset = 1;
			//key的长度，4字节
			System.arraycopy(intToByteArray(keyLen), 0, encode, offset, 4);
			offset += 4;
			//key的值
			System.arraycopy(encryptKey, 0, encode, offset, keyLen);
			offset += keyLen;

			//type heart，1字节, 0x03
			encode[offset] = 0x03;
			offset += 1;
			//heart的长度，4字节
			System.arraycopy(intToByteArray(heartLen), 0, encode, offset, 4);
			offset += 4;
			//heart的值
			System.arraycopy(encryptHeart, 0, encode, offset, heartLen);

			return encode;

		} catch (Exception e) {
			//Logger.e("Collector", "encrypt error", e);
		}
		return null;
	}

	public static byte[] intToByteArray(int i) {
		byte[] result = new byte[4];
		// 由高位到低位
		result[0] = (byte) ((i >> 24) & 0xFF);
		result[1] = (byte) ((i >> 16) & 0xFF);
		result[2] = (byte) ((i >> 8) & 0xFF);
		result[3] = (byte) (i & 0xFF);
		return result;
	}

	public static int byteArrayToInt(byte[] bytes) {
		int value = 0;
		// 由高位到低位
		for (int i = 0; i < 4; i++) {
			int shift = (4 - 1 - i) * 8;
			value += (bytes[i] & 0x000000FF) << shift;// 往高位游
		}
		return value;
	}
}
