package com.iflytek.cloud.msc.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import android.content.Context;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.util.http.HttpDownloadImpl;
import com.iflytek.cloud.msc.util.http.HttpDownloadListener;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.util.FileDownloadListener;
/**
 * 下载管理类，支持多任务同时下载
 */
public class FileDownloader {
	
	private static FileDownloader mFileDownload;
	/** 当前正在下载的资源信息 */
	private HashMap<Long,HashParam> mDownloadInfo ;
	private HashMap<Long,FileDownloadListener> mDownloadListener ;
	/** 临时保存下载文件路径*/
	private HashMap<Long, String> mFileTempPath;
	/** 下载请求 */
	private HttpDownloadImpl mHttpRequestImpl = null;
	private Context mContext;
	/** */
	private Config config = null;
	
	private final String DLOAD_URL = "download_uri";
	private final String DLOAD_PATH = "file_path";
	private final String DLOAD_MD5 = "file_md5";
	//Object Lock
	protected Object mSynObj = new Object();
	
	public static FileDownloader getDownLoadManager(Context context){
		if(mFileDownload == null){
			mFileDownload = new FileDownloader(context);
		}
		return mFileDownload;
	}
	
	private FileDownloader(Context context){
		mContext = context;
		mDownloadInfo = new HashMap<Long,HashParam>();
		mFileTempPath = new  HashMap<Long, String>();
		mDownloadListener = new HashMap<Long, FileDownloadListener>();
		config = Config.getConfig(mContext);
	}
	
	public int startDownload(String uri, String path, String md5, FileDownloadListener listener) {
		
		long httpId = getTaskId(uri, path, md5);
		if(mDownloadInfo.size()>0 && httpId != 0L) {
			mDownloadListener.put(httpId, listener);
			return -1;
		} else {
			httpId = System.currentTimeMillis();
		}
		
		if(listener != null) {
			mDownloadListener.put(httpId, listener);
		}
		
		HashParam value = new HashParam();
		value.putParam(DLOAD_URL, uri);
		value.putParam(DLOAD_PATH, path);
		value.putParam(DLOAD_MD5, md5);
		mDownloadInfo.put(httpId, value);
		
		// 如需要支持断点续传，这里要从sp中搜索临时文件路径传入
		String tempFile = config.getString(uri, null);
		DebugLog.LogD("tempFile:" + tempFile);
		
		//mHttpRequestImpl = new HttpDownloadImpl(httpId, 0, mContext);
		//mHttpRequestImpl.setHttpDownloadListener(httpDownloadListener);
		//mHttpRequestImpl.start(uri, tempFile, path, true, null);
		return ErrorCode.SUCCESS;
	}
	
	private HttpDownloadListener httpDownloadListener = new HttpDownloadListener() {

		@Override
		public void onStart(long length, String mimeType, String newPath,
				String eTag, HttpDownloadImpl download) {
			DebugLog.LogD("httpdownload onStart:length:"+length + " mimeType:"+mimeType + " newPath:"+newPath);
			
			if(mDownloadInfo == null) {
				download.cancel();
				return;
			}
			
			// 保存临时文件全路径
			mFileTempPath.put(download.getId(), newPath);
			
			HashParam param = mDownloadInfo.get(download.getId());
			config.putString(param.getString(DLOAD_URL), newPath);

			if(mDownloadListener.size() > 0 
					&& mDownloadListener.get(download.getId()) != null) {
				mDownloadListener.get(download.getId()).onStart();
			}
		}

		@Override
		public void onProgress(long currentBytes, int percent, HttpDownloadImpl download) {
			DebugLog.LogD("httpdownload onProgress:currentBytes:"+currentBytes + " percent:"+percent);

			if(mDownloadInfo == null) {
				download.cancel();
				return;
			}

			if(mDownloadListener.size() > 0 
					&& mDownloadListener.get(download.getId()) != null) {
				mDownloadListener.get(download.getId()).onProgress(percent);
			}
		}

		@Override
		public void onFinish(String fileFullPath, HttpDownloadImpl download) {
			DebugLog.LogD("httpdownload onFinish:fileName:"+fileFullPath);
//			String jsonStr = "{\"resourceKey\":\"12345\",\"size\":\"123.23k\",\"name\":\"芝麻开门\",\"type\":\"ivw\",\"status\":\"FREE\",\"vercode\":\"3.6\",\"uri\":\"http://szwu.hfdn.openstorage.cn/123/kdxf.irf\"}";
//			HashParam resParam = mDownloadInfo.get(download.getId());
//			resParam.putParam("path", fileFullPath);
			if(mDownloadInfo == null) {
				download.cancel();
				return;
			}
			
			// 删除临时资源路径
			if(mFileTempPath.size() >0){
				mFileTempPath.remove(download.getId());
			}

			String fileMD5 = mDownloadInfo
					.get(download.getId()).getString(DLOAD_MD5);
			// 删除当前保存的下载list
			if (mDownloadInfo.size() > 0) {
				config.removeBean(mDownloadInfo
						.get(download.getId()).getString(DLOAD_URL));
				mDownloadInfo.remove(download.getId());
			}

			// 保存到资源管理器中
			DebugLog.LogD("path="+fileFullPath);
			// 流程结束，返回信息，监听置空
			if(mDownloadListener.size() > 0 
					&& mDownloadListener.get(download.getId()) != null) {
				FileDownloadListener listener = mDownloadListener.get(download.getId());
				if(FileUtil.checkFileMD5(fileMD5, fileFullPath)) {
					DebugLog.LogD("this file calculate md5 success！");
					listener.onCompleted(fileFullPath, null);
				} else {
					listener.onCompleted(null, new SpeechError(ErrorCode.ERROR_INVALID_DATA));
					DebugLog.LogE("this file calculate md5 error!");
				}
				listener = null;
				mDownloadListener.remove(download.getId());
			}
		}

		@Override
		public void onError(int errorCode, HttpDownloadImpl download) {
			DebugLog.LogE("httpdownload onError:errorCode:"+errorCode);

			if(mDownloadInfo == null) {
				download.cancel();
				return;
			}
			
			if(mFileTempPath.size() >0){
				mFileTempPath.remove(download.getId());
			}
			
			if (mDownloadInfo.size() > 0) {
				mDownloadInfo.remove(download.getId());
			}
			// 流程结束，返回信息，监听置空
			if(mDownloadListener.size() > 0 
					&& mDownloadListener.get(download.getId()) != null) {
				mDownloadListener.get(download.getId()).onCompleted(null, new SpeechError(errorCode));
				mDownloadListener.remove(download.getId());
			}
		}
	};
	
	@SuppressWarnings("rawtypes")
	private long getTaskId(String uri, String path, String md5) {
		synchronized (mSynObj) {
			Iterator iter = mDownloadInfo.entrySet().iterator();
			while(iter.hasNext()) {
				Map.Entry entry = (Map.Entry) iter.next();
				long key = ((Long)entry.getKey()).longValue();
				HashParam val = (HashParam) entry.getValue();
				if(val.getString(DLOAD_URL).equals(uri)
						&& val.getString(DLOAD_PATH).equals(path)
						&& val.getString(DLOAD_MD5).equals(md5))
					return key;
			}
			return 0L;
		}
	}
}
