/*
 * 文件名：StateRecordingDrawer.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：录音识别动画，通过剪切MicColor所在canvas的大小，达到音量显示的效果；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.ui.control;

import com.iflytek.cloud.msc.util.log.DebugLog;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PaintFlagsDrawFilter;
import android.graphics.Path;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;

public class StateRecordingDrawer extends View{
	
	private Drawable mMicDrawable;
	private Drawable mMicColor;
	private PaintFlagsDrawFilter mPaint = new PaintFlagsDrawFilter(
			Paint.ANTI_ALIAS_FLAG, Paint.FILTER_BITMAP_FLAG);
	Path mPath;
	
	/**
	 * 初始化识别动画，该接口留于实现自定义View使用.
	 * @param context 上下文环境
	 * @param attrs  View的相关属性
	 */
	public StateRecordingDrawer(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	/**
	 * 初始化识别界面元素
	 * @param context
	 */
	public StateRecordingDrawer(Context context) {
		super(context);
		
		try {
			mMicDrawable = DrawableMgr.getDrawable(getContext(), DrawableMgr
					.TAG_RECORDING_ANIMATION + "empty");
			
			mMicColor = DrawableMgr.getDrawable(getContext(), DrawableMgr
					.TAG_RECORDING_ANIMATION + "full");
			
			mMicDrawable.setBounds(new Rect(
					-mMicDrawable.getIntrinsicWidth()  / 2,
					-mMicDrawable.getIntrinsicHeight() / 2, 
					 mMicDrawable.getIntrinsicWidth()  / 2, 
					 mMicDrawable.getIntrinsicHeight() / 2));
			
			mMicColor.setBounds(new Rect(
					-mMicColor.getIntrinsicWidth()  / 2,
					-mMicColor.getIntrinsicHeight() / 2, 
					 mMicColor.getIntrinsicWidth()  / 2, 
					 mMicColor.getIntrinsicHeight() / 2));
			
			mPath = new Path();
			setVolume(0);
		}catch(Exception e) {
			DebugLog.LogE(e);
		}
	}

	/**
	 * 设置音量，根据音量设置绘制的Canvas大小
	 * @param volume 音量
	 */
	public void setVolume(int volume) {

		mPath.reset();
		mPath.addCircle(0, 0, ( mMicDrawable.getIntrinsicWidth() * volume ) / 12
			, Path.Direction.CCW);
	}
	
	/**
	 * 绘制识别动画
	 * @param canvas 画布
	 */
	public void onDraw(Canvas canvas) {
		canvas.save();
		/*去除锯齿*/
		canvas.setDrawFilter(mPaint);
		
		/* 移到中心点*/
		canvas.translate(getWidth() / 2, getHeight() / 2);
		mMicDrawable.draw(canvas);	
		
		/* 剪切画布大小*/
		canvas.clipPath(mPath);
		mMicColor.draw(canvas);
		canvas.restore();
	}

	/**
	 * 销毁资源
	 */
	public void finalize() throws Throwable {
		mMicDrawable = null;
		mMicColor = null;
		
		super.finalize();
	}
	
	/**
	 * 设置View大小
	 */
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		 super.onMeasure(widthMeasureSpec, heightMeasureSpec);

		int width = MeasureSpec.getSize(widthMeasureSpec);
		int height = MeasureSpec.getSize(heightMeasureSpec);
		
		Drawable drawable = getBackground();
		if (drawable != null) {
			width = drawable.getMinimumWidth();
			height = drawable.getMinimumHeight();
		}		
		setMeasuredDimension(resolveSize(width, widthMeasureSpec),
				resolveSize(height, heightMeasureSpec));
	}
}
