package com.iflytek.cloud.util.code.impl;

import java.util.HashMap;
import java.util.Map;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.util.AudioDetector;
import com.iflytek.msc.MetaVAD;

import android.content.Context;
import android.text.TextUtils;

public class AudioDetectorImplMetaVad extends AudioDetector {
	private static final float DEF_THRESHOLD = 0.6f;
	private static final int DEF_MAX_REPEAT_SEG = 10;
	private static final int METAVAD_MAX_VOL = 7;
	private static final int VOL_SCALE = VOLUME_MAX / METAVAD_MAX_VOL;
	
	private static final Map<String, String> sName2Key = new HashMap<String, String>();
	private static final Map<String, String> sName2Def = new HashMap<String, String>();
	
	private HashParam mParams = new HashParam();
	private final DetectorResult mResult = new DetectorResult();
	private MetaVAD.Instance mVadInstance = new MetaVAD.Instance();
	
	private String mCharset = DataUtil.GB2312;
	private boolean mHasSubStart = false;
	private boolean mIsFirstSub = true;
	private int mSegRepeatCount = 0;
	
	private int mStart = 0;
	
	private static final long VAL_INFINITE = -1;
	private int mBytesOfPerSample = 2;
	private long mBytesOfSpeechTimeout = VAL_INFINITE;
	private long mBytesOfSpeech = 0;

	static{
		//初始化map表
		sName2Key.put( SpeechConstant.VAD_BOS, "vad_starttimeout" );
		sName2Key.put( SpeechConstant.VAD_EOS, "vad_endtimeout" );
		sName2Key.put( THRESHOLD, "vad_threshold" );
		
		sName2Def.put( SpeechConstant.VAD_BOS, String.valueOf(DEF_BOS) );
		sName2Def.put( SpeechConstant.VAD_EOS, String.valueOf(DEF_EOS) );
		sName2Def.put( THRESHOLD, String.valueOf(DEF_THRESHOLD) );
	}
	
	static public class Status{
		public final static int SUCCEES = 0x0000;
		public final static int FAILED = 0x1000;
		
		public final static int START = 0x01;
		public final static int SUBSTART = 0x02;
		public final static int SUBEND = 0x03;
		public final static int END = 0x04;
		public final static int HASSEG = 0x05;
		public final static int NOSEG = 0x06;
	}

	public AudioDetectorImplMetaVad(Context context, String param) {
		super(context, param);
		DebugLog.LogD( "Meta VAD AudioDetector constructor enter, context: "+context+", param: "+param );
		
		mParams.initWithParam( param );
		try{
			mCharset = mParams.getString( SpeechConstant.TEXT_ENCODING, mCharset );
			final String extra = mParams.getString( EXTRA );
			byte[] extraBytes = null;
			if( null != extra ){
				extraBytes =  DataUtil.getNativeByteArray( extra, mCharset );
			}
			
			DebugLog.LogD( "MetaVAD.VADInitialize begin." );
			int ret = MetaVAD.VADInitialize( extraBytes  );
			
			if( Status.SUCCEES == ret ){
				mVadInstance.rate = mParams.getInt( SpeechConstant.SAMPLE_RATE, DEF_RATE );
				final String resPath = mParams.getString( RES_PATH );
				byte[] resPathBytes = null;
				if( null != resPath ){
					resPathBytes = DataUtil.getNativeByteArray( resPath, mCharset );
				}
				
				DebugLog.LogD( "MetaVAD.VADLoadResource begin." );
				ret = MetaVAD.VADLoadResource( mVadInstance.rate, resPathBytes );
				
				if( Status.SUCCEES == ret ){
					DebugLog.LogD( "MetaVAD.VADCreateSession begin." );
					ret = MetaVAD.VADCreateSession( mVadInstance );
				}//end of if VADLoadResource SUCCEES
			}//end of if VADInitialize SUCCEES
			
			if( Status.SUCCEES  != ret ){
				DebugLog.LogE( "MetaVAD Native error "+ret );
			}
		}catch( Throwable e ){
			DebugLog.LogE( "Meta VAD AudioDetector constructor exception:" );
			DebugLog.LogE( e );
		}//end of try-catch
		
		DebugLog.LogD( "Meta VAD AudioDetector constructor leave" );
	}

	@Override
	public boolean destroy() {
		DebugLog.LogD( "destroy enter" );
		boolean ret = true;
		
		synchronized( sSyncObj ){
			try{
				int nativeRet = Status.SUCCEES;;
				if( null != mVadInstance ){
					if(MetaVAD.Instance.INVALID_HANDLE!=mVadInstance.handle ){
						DebugLog.LogD( "destroy MetaVAD.VADDestroySession begin" );
						nativeRet = MetaVAD.VADDestroySession( mVadInstance );
						DebugLog.LogD( "destroy MetaVAD.VADDestroySession ret="+nativeRet );
					}// end of handle is valid
					
					if( Status.SUCCEES == nativeRet ){
						mVadInstance.handle = MetaVAD.Instance.INVALID_HANDLE;
						
						DebugLog.LogD( "destroy MetaVAD.VADDelResource begin" );
						nativeRet = MetaVAD.VADDelResource( mVadInstance.rate );
						DebugLog.LogD( "destroy MetaVAD.VADDelResource ret="+nativeRet );
						
						DebugLog.LogD( "destroy MetaVAD.VADUninitialize begin" );
						nativeRet = MetaVAD.VADUninitialize();
						DebugLog.LogD( "destroy MetaVAD.VADUninitialize ret="+nativeRet );
					}
					
					ret = Status.SUCCEES==nativeRet;
					if( ret ){
						mVadInstance = null;
						sInstance = null;
					}
				}//end of if vad instance not null
			}catch( Throwable e ){
				DebugLog.LogE( "destroy exception:" );
				DebugLog.LogE( e );
				ret = false;
			}//end of try-catch
		}//end of synchronized
		
		DebugLog.LogD( "destroy leave: "+ret );
		return ret;
	}

	@Override
	public void reset() {
		DebugLog.LogD( "reset enter" );
		
		synchronized( sSyncObj ){
			if( null!=mVadInstance
					&& MetaVAD.Instance.INVALID_HANDLE!=mVadInstance.handle ){
				try{
					DebugLog.LogD( "reset MetaVAD.VADResetSession begin" );
					final int ret = MetaVAD.VADResetSession( mVadInstance );
					DebugLog.LogD( "reset MetaVAD.VADResetSession return "+ret );
					mVadInstance.reset();
					
					mIsFirstSub = true;
					mHasSubStart = false;
					mBytesOfSpeech = 0;
					mStart = 0;
				}catch( Throwable e ){
					DebugLog.LogE( "reset exception:" );
					DebugLog.LogE( e );
				}//end of try-catch
			}else{
				DebugLog.LogE( "setParameter error: vad instance is null, or invalid handle." );
			}//end of if-else vad instance not null and handle is valid
		}//end of synchronized
		
		DebugLog.LogD( "reset leave" );
	}

	@Override
	public DetectorResult detect(byte[] buffer, int offset, int length, boolean isLast) {
		DebugLog.LogI( "detect enter, buffer: "+buffer+", offset: "+offset+", length: "+length+", isLast: "+isLast );
		
		synchronized( sSyncObj ){
			try{
				this.resetResult();
				
				if( null==mVadInstance 
						|| MetaVAD.Instance.INVALID_HANDLE==mVadInstance.handle ){
					DebugLog.LogE( "detect error: vad instance null, or handle is invalid!" );
					mResult.error = ErrorCode.ERROR_ENGINE_INIT_FAIL;
				}else if( null==buffer 
						|| 0>=length 
						|| MAX_BUF_LEN<length
						|| 0>offset
						|| buffer.length-offset<length){
					 if( !isLast ){
						 mResult.error = ErrorCode.ERROR_INVALID_PARAM;
					 }//end of if is last
				}else{
					DebugLog.LogI( "detect buffer length: "+length );
				}// end of if parameter is valid
				
				if( Status.SUCCEES == mResult.error ){
					final int last = isLast ? 1 : 0;
					int ret = MetaVAD.VADAppendPCM( mVadInstance, buffer, offset, length, last );
					DebugLog.LogI( "MetaVAD VADAppendPCM ret: "+ ret );
					
					//Count for current bytes of speech since the next detect call of find start
					if( mHasSubStart ){
						mBytesOfSpeech += length;
					}
					
					updateStatus( ret );
					
					if( ErrorCode.SUCCESS == mResult.error ){
						ret = Status.HASSEG;	//set ret value to has seg, so the while block can run 
						while( Status.HASSEG == ret ){
							ret = MetaVAD.VADGetSeg( mVadInstance );
							DebugLog.LogI( "MetaVAD VADGetSeg ret: "+ ret
									+", seg status: "+mVadInstance.seg
									+", seg begin: "+mVadInstance.begin
									+", seg end: "+mVadInstance.end );
							
							updateStatus( ret );
							if( ErrorCode.SUCCESS == mResult.error ){
								updateResult();
								
								//return users audio data
								mResult.buffer = buffer;
								mResult.length = length;
								mResult.offset = offset;
							}//end of if is success
							
							if( MetaVAD.Instance.SEG_LAST == mVadInstance.seg
									|| ErrorCode.SUCCESS != mResult.error ){
								DebugLog.LogD( "detect get last seg or error." );
								break;
							}//end of if is last seg or error occur
						}//end of while
					}//end of if is success
				}// end of append data
			}catch( UnsatisfiedLinkError  e){
				DebugLog.LogE( "detect exception" );
				DebugLog.LogE( e );
				this.resetResult();
				mResult.error = ErrorCode.ERROR_UNSATISFIED_LINK;
			}catch( Throwable e ){
				DebugLog.LogE( "detect exception" );
				DebugLog.LogE( e );
				this.resetResult();
				mResult.error = ErrorCode.ERROR_UNKNOWN;
			}//end of try-catch
		}//end of synchronized
		
		DebugLog.LogI( "detect leave" );
		return mResult;
	}

	@Override
	public void setParameter(String name, String value) {
		DebugLog.LogD( "setParameter enter, key: "+name+", value: "+value );
		
		synchronized( sSyncObj ){
			if( null!=mVadInstance
					&&MetaVAD.Instance.INVALID_HANDLE!=mVadInstance.handle ){
				try{
					if( RESET_SENTENCE.equalsIgnoreCase(name) ){
						int ret = MetaVAD.VADResetSentence( mVadInstance );
						DebugLog.LogD( "VAD VADResetSentence ret: "+ret );
					}else if( !TextUtils.isEmpty(name) && sName2Key.containsKey(name) ){
						if( !TextUtils.isEmpty(value) ){
							mParams.putParam( name, value );
						}else{
							mParams.remove( name );
						}//end of if-else is value not empty
						
						final String newValue = mParams.getString( name, sName2Def.get(name) );
						final String keyValue = sName2Key.get( name );
						int ret = MetaVAD.VADSetParam( mVadInstance
								, DataUtil.getNativeByteArray(keyValue, mCharset)
								, DataUtil.getNativeByteArray(newValue, mCharset) );
						DebugLog.LogD( "VAD SetParameter key="+keyValue+", value="+newValue+", ret: "+ret );
					}else if( !TextUtils.isEmpty(name) ){
						if( SpeechConstant.KEY_SPEECH_TIMEOUT.equalsIgnoreCase(name) ){
							long timeout = VAL_INFINITE;
							try{
								timeout = Long.parseLong( value );
							}catch( NumberFormatException e ){
								DebugLog.LogE( e );
							}
							
							DebugLog.LogD( "SetParameter speech timeout value:"+timeout );
							if( 0<timeout ){
								mBytesOfSpeechTimeout = mVadInstance.rate * mBytesOfPerSample * timeout / 1000;
								DebugLog.LogD( "SetParameter BytesOfSpeechTimeout: "+mBytesOfSpeechTimeout );
							}else{
								mBytesOfSpeechTimeout = VAL_INFINITE;
							}
						}else{
							int ret = MetaVAD.VADSetParam( mVadInstance
									, DataUtil.getNativeByteArray(name, mCharset)
									, DataUtil.getNativeByteArray(value, mCharset) );
							DebugLog.LogD( "VAD SetParameter name="+name+", value="+value+", ret: "+ret );
						}//end of if-else jar parameter
					}//end of if-else is key not empty
				}catch( Throwable e ){
					DebugLog.LogE( "setParameter exception" );
					DebugLog.LogE( e );
				}//end of try-catch
			}else{
				DebugLog.LogE( "setParameter error: vad instance is null, or invalid handle." );
			}//end of if-else handle is valid
		}//end of synchronized
		
		DebugLog.LogD( "setParameter leave." );
	}

	private void resetResult(){
		mResult.buffer = null;
		mResult.end = 0;
		mResult.error = ErrorCode.SUCCESS;
		mResult.length = 0;
		
		mResult.offset = 0;
		mResult.quality = 0;
		mResult.start = 0;
		mResult.status = DetectorResult.STATUS_OK;
		
		mResult.sub = DetectorResult.SUB_OK;
		mResult.subs.clear();
		mResult.voice = false;
		
		mResult.volume = 0;
		mResult.confidence = DEF_CONFIDENCE;
		
		if( null != mVadInstance ){
			mVadInstance.reset();
		}
		
		mSegRepeatCount = 0;
	}

	private void updateResult(){
		if( MetaVAD.Instance.SEG_NONE != mVadInstance.seg ){
			Integer oldKey = mResult.subs.put( mVadInstance.begin, mVadInstance.end );
			if( null != oldKey ){
				DebugLog.LogE( "update result error: repeat sub begin: "+ oldKey );
				if( DEF_MAX_REPEAT_SEG <= ++mSegRepeatCount ){
					mResult.error = ErrorCode.MSP_ERROR_GENERAL;
					DebugLog.LogE( "update result error: repeat sub reach max count." );
				}
			}//end of if repeat
			mResult.sub = DetectorResult.SUB_STARTEND;
			
			if( MetaVAD.Instance.SEG_FIRST == mVadInstance.seg
					|| (mIsFirstSub && MetaVAD.Instance.SEG_LAST==mVadInstance.seg) ){
				mResult.start = mVadInstance.begin;
				mStart = mResult.start;
			}
			
			if( MetaVAD.Instance.SEG_LAST == mVadInstance.seg ){
				mResult.end =mVadInstance.end;
				mResult.start = mStart;
				mResult.confidence = MetaVAD.VADGetSentConfidence(mVadInstance);
			}//end of if-else is first or last seg
			
			mIsFirstSub = false;
		}//end of if has seg
		
		mResult.quality = 0;
		mResult.voice = false;
		mResult.volume = mVadInstance.volume * VOL_SCALE;
	}
	
	/*
	 * <date>2016-01-27</date>
	 * <author>gxlin</author>
	 * <p>
	 * 		已与引擎确认，其中引擎的 SUBSTART 和 SUBEND 目前
	 * 暂没有用到，分句信息以 VAD_HAS_SEG 为准。参考对
	 *  DetectorResult.sub 的状态值说明。
	 *  	MetaVAD 的 前端点（VAD_FINDSTART）与FixFront
	 *  不一样，MetaVAD前端点指找到音频开始位置，而BOS的值
	 *  与EOS值一样，为VAD_FINDEND。所以，目前通过记录是否
	 *  当前会话有分句开始信息( mHashSubStart )来区分是前端点，
	 *  还是后端点。故，每次应用写入的音频长度，不能太长，否则
	 *  如果一次音频就同时找到了 VAD_FINDSTART 和 VAD_FINDEND，
	 *  则可能导致本应的后端点的，错返回了前端点。
	 * </p>
	 */
	private void updateStatus( int status ){
		switch( status ){
		case Status.SUCCEES:
		case Status.NOSEG:
			mResult.error = ErrorCode.SUCCESS;
			mVadInstance.seg = MetaVAD.Instance.SEG_NONE;
			break;
		case Status.START:
		case Status.SUBSTART:
			mResult.sub = DetectorResult.SUB_START;
			break;
		case Status.SUBEND:
			mResult.sub = DetectorResult.SUB_END;
			break;
		case Status.END:
			mResult.status = mHasSubStart ? DetectorResult.STATUS_EOS : DetectorResult.STATUS_BOS;
			break;
		case Status.HASSEG:
			mResult.sub = DetectorResult.SUB_STARTEND;
			break;
		case Status.FAILED:
		default:
			mResult.error = status;
		}//end of switch
		
		//有可能首次就返回 sub_end，这时也应作为找到前端点
		if( !mHasSubStart
				&& DetectorResult.SUB_OK!=mResult.sub ){
			mHasSubStart = true;
			
			if( DetectorResult.STATUS_OK==mResult.status ){
				mResult.status = DetectorResult.STATUS_START;
			}
		}//end of if find first sub
		
		//Checking whether speech timeout or not
		if( DetectorResult.STATUS_OK==mResult.status
				&& isSpeechTimeout() ){
			mResult.status = DetectorResult.STATUS_TIMEOUT;
		}
	}//end of function updateStatus

	private boolean isSpeechTimeout(){
		boolean ret = false;
		if( 0 < mBytesOfSpeechTimeout ){
			ret = mBytesOfSpeechTimeout<=mBytesOfSpeech;
		}
		
		return ret;
	}//end of function isSpeechTimeout
}
