package com.iflytek.cloud.util.contacts.impl;

import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.CommonDataKinds.StructuredName;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.PhoneLookup;
import android.provider.ContactsContract.RawContacts;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.util.contacts.ContactManagerImpl;
import com.iflytek.cloud.util.contacts.PhoneNumberUtil;
import com.iflytek.cloud.util.contacts.interfaces.ContactAccessor;

public class ContactAccessorSdk5 extends ContactAccessor {
    
    private static final String[] CONTACT_PROJECTION_20 = new String[] {     
        ContactsContract.Contacts.DISPLAY_NAME,
        ContactsContract.Contacts._ID};
    
    private static final String[] CONTACT_INFO_PROJECTION_20 = new String[] {     
    	Phone.DISPLAY_NAME, 
    	Phone.NUMBER, 
    	Phone.CONTACT_ID }; 
    
    private static final String[] CONTACT_ID_PROJECTION_20 = new String[] {
        ContactsContract.Contacts._ID, 
        ContactsContract.Contacts.HAS_PHONE_NUMBER};
    
    private static final String[] ID_PROJECTION_20 = new String[] {
        ContactsContract.CommonDataKinds.Phone.CONTACT_ID};
    
    private static final String[] LOOKUP_PROJECTION_20 = new String[] {
        PhoneLookup.DISPLAY_NAME};
    
    private static final String[] CONTACT_ITEM_PROJECTION_20 = new String[] {
        ContactsContract.CommonDataKinds.Phone.NUMBER,
        ContactsContract.CommonDataKinds.Phone.TYPE,
        ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME
        };
    
    private static final String[] CONTACT_HAS_NUMBER_PROJECTION_20 = new String[] {
        ContactsContract.Contacts.HAS_PHONE_NUMBER};
    
    public ContactAccessorSdk5(Context context) {
        super(context);
        setNumberType(context);
    }
    
    @Override
    public Uri getContactUri() {
        return ContactsContract.Contacts.CONTENT_URI;
    }

    @Override
    public String queryContactIdByNumber(String number) {
        Cursor cursor = null;
        String contactId = null;
        String contactWhere = ContactsContract.CommonDataKinds.Phone.NUMBER + " like " + "'%" + number + "'";
        
        try {
            if (null == number) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactId_20----id is null");
            } else {
                cursor = mContext.getContentResolver().query(
                        ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
                        ID_PROJECTION_20, contactWhere, null, null);
                if (null == cursor) {
                	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactId_20--------cursor is null");
                } else if (0 == cursor.getCount()) {
                	DebugLog.LogD(ContactManagerImpl.TAG, "queryContactId_20--------cursor count is 0");
                } else {
                    while (cursor.moveToNext()) {
                        contactId = cursor.getString(0);
                        break;
                    }
                }
            }
            return contactId;
        } catch (Exception e) {
            DebugLog.LogE(e);
            return contactId;
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
    }

    @Override
    public ArrayList<String> queryPhoneNumberByName(String name) {
    	DebugLog.LogD(ContactManagerImpl.TAG, "-------------queryPhoneNumber-------------");
        Cursor contactCursor = null;
        Cursor phoneCursor = null;
        String contactId = null;
        String phoneNumber = null;
        String contactsWhere = ContactsContract.Contacts.DISPLAY_NAME + " = " + name;
        String phoneWhere = ContactsContract.CommonDataKinds.Phone.CONTACT_ID + " = " + contactId;
        ArrayList<String> tmpList = new ArrayList<String>();
        
        try {
            contactCursor = mContext.getContentResolver().query(
                    ContactsContract.Contacts.CONTENT_URI, 
                    CONTACT_ID_PROJECTION_20, contactsWhere, null, getOrderByKey());
            if (null == contactCursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_20-------contactCursor is null");
            } else if (0 == contactCursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_20-------contactCursor getCount == 0");
            } else if (contactCursor.moveToFirst()) {
                contactId = contactCursor.getString(
                        contactCursor.getColumnIndex(ContactsContract.Contacts._ID));
                if (contactCursor.getInt(
                        contactCursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER)) > 0) {
                    phoneCursor = mContext.getContentResolver().query(
                            ContactsContract.CommonDataKinds.Phone.CONTENT_URI, 
                            null, phoneWhere, null, getOrderByKey());
                    if (null == phoneCursor) {
                    	DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_20-------phoneCursor is null");
                    } else if (0 == phoneCursor.getCount()) {
                    	DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_20-------phoneCursor getCount == 0");
                    } else {
                        while (phoneCursor.moveToNext()) {
                        	phoneNumber = phoneCursor.getString(
                                    phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER));
                            tmpList.add(phoneNumber);
                        }
                        DebugLog.LogD(ContactManagerImpl.TAG, "queryPhoneNumberByName_20-------count = "  + phoneCursor.getCount());
                    }
                }
            }
            
            return tmpList;
        } catch (Exception e) {
            DebugLog.LogE(e);
            return null;
        } finally {
            if (null != contactCursor) {
                contactCursor.close();
            }
            if (null != phoneCursor) {
                phoneCursor.close();
            }
        }
    }
    @Override
    public void insertIntentContact(String name, String number) {
        try {
            Uri insertUri = ContactsContract.Contacts.CONTENT_URI;
            Intent intent = new Intent(Intent.ACTION_INSERT, insertUri);
            if (null != name) {
                intent.putExtra(ContactsContract.Intents.Insert.NAME, name);
            }
            if (null != number) {
                intent.putExtra(ContactsContract.Intents.Insert.PHONE, number);
            }
            intent.putExtra(ContactsContract.Intents.Insert.PHONE_TYPE,
                  ContactsContract.CommonDataKinds.Phone.TYPE_MOBILE);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK
                  | Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET);
            mContext.startActivity(intent);
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
    }


    @Override
    public boolean hasContactNumber(String name) {
        Cursor cursor = null;
        boolean hasNumberFlag = false;
        String where = ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME + " = ?";
        String[] whereParams = new String[] {name};
        
        try {
            cursor = mContext.getContentResolver().query(ContactsContract.Contacts.CONTENT_URI, 
                    CONTACT_HAS_NUMBER_PROJECTION_20, where, whereParams, getOrderByKey());
            if (null == cursor) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "has_contact_number_20 ----------------------- cursor is null");
                return hasNumberFlag;
            } else if (0 == cursor.getCount()) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "has_contact_number_20 ----------------------- cursor size is 0");
                return hasNumberFlag;
            } else if (cursor.moveToNext()) {
                if (cursor.getInt(cursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER)) > 0) {
                    hasNumberFlag = true;
                }
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        } finally {
            if (null != cursor) {
                cursor.close();
            }
        }
        return hasNumberFlag;
    }

    @Override
    protected String[] getContactProjection() {
        return CONTACT_PROJECTION_20;
    }
    
    @Override
    protected String[] getContactInfoProjection() {
        return CONTACT_INFO_PROJECTION_20;
    }
    
    @Override
    protected String[] getLookUpProjection() {
        return LOOKUP_PROJECTION_20;
    }

    @Override
    protected String getOrderByKey() {
        if (Integer.parseInt(android.os.Build.VERSION.SDK) >= android.os.Build.VERSION_CODES.FROYO) {
            return "sort_key";
        } else {
            return ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME;
        }
    }

    @Override
    protected Uri getLookUpUri() {
        return ContactsContract.PhoneLookup.CONTENT_FILTER_URI;
    }

    @Override
    protected Uri getContactFilterUri() {
        return ContactsContract.PhoneLookup.CONTENT_FILTER_URI;
    }

    @Override
    public Uri getPhoneUri() {
        return ContactsContract.CommonDataKinds.Phone.CONTENT_URI;
    }

    @Override
    public String[] getPhoneProjection() {
        return CONTACT_ITEM_PROJECTION_20;
    }

    @Override
    public String getContactSetWhereStr(boolean fuzzyFlag) {
        if (fuzzyFlag) {
            return ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME + " like ?";
        }
        return ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME + " = ?";
    }

    @Override
    public String[] getContactSetWhereParams(String name, boolean fuzzyFlag) {
        if (fuzzyFlag) {
            return new String[] {"%" + PhoneNumberUtil.filterSqlName(name) + "%"};
        }
        return new String[] {name};
    }

	@Override
	public byte[] queryContactPhotoDataByName(String name) {
		Cursor cursor = null;
		byte[] contactTmp = new byte[512];
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		InputStream inputStream = null;
		
		String[] projection = new String[] {Photo.PHOTO_ID, Phone.CONTACT_ID};
		try {
			  cursor = mContext.getContentResolver().query(getPhoneUri(), projection, getContactSetWhereStr(false), 
	                    getContactSetWhereParams(name, false), null);
			  if (null == cursor) {
				  DebugLog.LogD(ContactManagerImpl.TAG, "queryContactPhoto ----------------cursor is null");
	              return null;
			  } else if (0 == cursor.getCount()) {
				  DebugLog.LogD(ContactManagerImpl.TAG, "queryContactPhoto ----------------cursor size is 0");
	              return null;
			  } else {
				  if (cursor.moveToFirst()) {
					  long photoId = cursor.getLong(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Photo.PHOTO_ID));
					  long contactid = cursor.getLong(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.CONTACT_ID));
					  if (photoId > 0) {
						  Uri uri = ContentUris.withAppendedId(ContactsContract.Contacts.CONTENT_URI, contactid);  
					      inputStream = ContactsContract.Contacts.openContactPhotoInputStream(
					    		  mContext.getContentResolver(), uri);
					      int bytesRead = inputStream.read(contactTmp); 
					      while (bytesRead != -1)   { 
					    	  out.write(contactTmp, 0, bytesRead); 
					    	  bytesRead = inputStream.read(contactTmp); 
					      } 
					  }
				  }
				  if (null != inputStream) {
					 inputStream.close();
				  }
				  return out.toByteArray(); 
			  }
 		} catch (Exception e) {
			DebugLog.LogE(e);
			return null;
		} finally {
			if (null != cursor) {
				cursor.close();
			}
		}
	}

	@Override
	public Uri insertNewContact(String name, String number) {
		try {
			// 首先插入空值，再得到rawContactsId ，用于下面插值
			ContentValues values = new ContentValues();
			// insert a null value
			Uri rawContactUri = mContext.getContentResolver().insert(
					RawContacts.CONTENT_URI, values);
			long rawContactsId = ContentUris.parseId(rawContactUri);

			// 往刚才的空记录中插入姓名
			values.clear();
			values.put(Data.RAW_CONTACT_ID, rawContactsId);
			values.put(Data.MIMETYPE, StructuredName.CONTENT_ITEM_TYPE);
			values.put(StructuredName.GIVEN_NAME, name);
			mContext.getContentResolver().insert(Data.CONTENT_URI, values);
			
			// 插入电话
			values.clear();
			values.put(Data.RAW_CONTACT_ID, rawContactsId);
			values.put(Data.MIMETYPE, Phone.CONTENT_ITEM_TYPE);
			values.put(Phone.NUMBER, number);
			values.put(Phone.TYPE, Phone.TYPE_MOBILE);
			return mContext.getContentResolver().insert(Data.CONTENT_URI, values);
		} catch (Exception e) {
			DebugLog.LogE(e);
			return null;
		} 
	}

}
